﻿Type=StaticCode
Version=2.2
B4J=true
@EndOfDesignText@
' Utilities module
' ###########################################################################################################################
' This module is a part of One Click Around application : 	http://oneclickaround.esy.es/
' One Click Around forum : 		http://oneclickaround.esy.es/Forum

' Copyright (C) 2014 by Didier Terrien, France
' ###########################################################################################################################

Sub Process_Globals
	Private fx As JFX
	Private Last_file_choosed As String
	Private Last_folder_choosed As String
End Sub

#Region  == Interface functions =======================================================================
Public Sub Set_form_fullScreen(frm As Form)
  Dim ps As Screen = fx.PrimaryScreen
  frm.WindowLeft = ps.MinX
  frm.WindowWidth = ps.MaxX - ps.MinX
  frm.WindowTop = ps.MinY
  frm.WindowHeight = ps.MaxY - ps.MinY
End Sub

Public Sub Get_mouse_absolute_x_position (EventData As MouseEvent) As Double 
Dim jo As JavaObject = EventData
	Return jo.RunMethod("getScreenX", Null)
End Sub

Public Sub Get_mouse_absolute_y_position (EventData As MouseEvent) As Double
Dim jo As JavaObject = EventData
	Return jo.RunMethod("getScreenY", Null)
End Sub

Public Sub Get_mouse_button (EventData As MouseEvent) As String 
Dim jo As JavaObject = EventData
	Return jo.RunMethod("getButton", Null)
End Sub

Public Sub Set_image_opacity (Image As ImageView, Value As Double)
	Try
		Dim joImage As JavaObject = Image
		joImage.RunMethod ("setOpacity", Array As Object(Value))
	Catch
		Log("Set_opacity error : " & LastException.Message)
	End Try
End Sub

Public Sub Set_panel_opacity (This_panel As Pane, Value As Double)
	Try
		Dim joPane As JavaObject = This_panel
		joPane.RunMethod ("setOpacity", Array As Object(Value))
	Catch
		Log("Set_opacity error : " & LastException.Message)
	End Try
End Sub

Public Sub Bring_to_front (This_object As Object)
    Dim jo As JavaObject = This_object
    jo.RunMethod("toFront", Null)
End Sub
	
Public Sub Send_to_background (This_object As Object)
    Dim jo As JavaObject = This_object
    jo.RunMethod("toBack", Null)
End Sub

Public Sub Init_default_style (Current_style As String) As String 
	If Current_style.trim.length = 0 Then
		Dim Default_style As StringBuilder : Default_style.Initialize
		Default_style.Append("-fx-background-color: linear-gradient(#f2f2f2, #d6d6d6), linear-gradient(#fcfcfc 0%, #d9d9d9 20%, #d6d6d6 100%),linear-gradient(#dddddd 0%, #f6f6f6 50%);")
		Default_style.Append("-fx-background-radius: 8,7,6;")
		Default_style.Append("-fx-background-insets: 0,1,2;")
		Default_style.Append("-fx-text-fill: #666666;")
		Default_style.Append("-fx-effect: dropshadow( three-pass-box , rgba(0,0,0,0.7) , 5, 0.0 , 0 , 1 );")
		Default_style.Append("-fx-padding:5;")
		Return Default_style
	Else
		Return Current_style
	End If
End Sub
#End Region
	
#Region  == Strings functions =========================================================================
' See also : http://www.basic4ppc.com/android/forum/threads/jstringfunctions-beta.36595/#content
Public Sub String_list_to_array (lstArgs As List) As String()
    Dim arrArgs(lstArgs.Size) As String
    For I = 0 To arrArgs.Length - 1
        arrArgs(I) = lstArgs.Get(I)
    Next
    Return arrArgs
End Sub

Public Sub Right (Text As String, Caracters_number As Int) As String
	Return Text.SubString (Text.Length - Min (Caracters_number, Text.Length))
End Sub

Public Sub Left (Text As String, Caracters_number As Int) As String
	Return Text.SubString2 (0, 1)
End Sub
#End Region 

#Region  == Files functions ===========================================================================
' Get the files number recursively
Public Sub Get_file_number_recursive (Folder As String, Folder_size As Int) As Int 
	Dim File_list = File.ListFiles (Folder) As List
	For Each File_name As String In File_list
		If File.IsDirectory (Folder, File_name) Then
			Folder_size = Get_file_number_recursive (Folder & "\" & File_name, Folder_size)
		Else
			Folder_size = Folder_size + 1
		End If
	Next
	Return Folder_size
End Sub

' Get the files and directories recursively from a path
Public Sub Get_file_list_recursive (Folder As String, File_list As List) As List 
	Dim Folder_file_list = File.ListFiles (Folder) As List
	For Each File_name As String In Folder_file_list
		If File.IsDirectory (Folder, File_name) Then
			File_list.Add(Folder & "\" & File_name)
			Dim Sub_folder_file_list As List : Sub_folder_file_list.Initialize 
			File_list.AddAll(Get_file_list_recursive (Folder & "\" & File_name, Sub_folder_file_list))
		Else
			File_list.Add(Folder & "\" & File_name)
		End If
	Next
	Return File_list
End Sub

' Get relative path by removing reference path from absolute path
Public Sub Get_relative_path (Absolute_path As String, Reference_path As String) As String 
	Dim Full_Absolute_path = Get_long_file_name (Absolute_path.Trim) As String 
	Dim Full_Reference_path = Get_long_file_name (Reference_path.Trim) As String
	If  Full_Absolute_path.StartsWith(Full_Reference_path) Then
		Dim Relative_path = Absolute_path.SubString (Full_Reference_path.Length) As String 
		If Relative_path = "" Then
		Else If Relative_path.SubString2 (0, 1) = "\" Then 
			Relative_path = Relative_path.SubString (1)
		End If
		Return Relative_path
	Else
		Return ""
	End If
End Sub

' Get the long file name of the absolute path made by concatenating reference path with relative path
' If relative path is an absolute path, long file name of relative path is returned
Public Sub Get_absolute_path (Relative_path As String, Reference_path As String) As String 
	Dim Absolute_path As String 
	If Is_relative_path (Relative_path) Then
		If Left (Relative_path.Trim, 1) = "\" Then 
			Relative_path = Relative_path.Trim.SubString (1)
		End If
		If Right (Reference_path.Trim, 1) = "\" Then 
			Reference_path = Left (Reference_path.Trim, Reference_path.Trim.Length - 1)
		End If
		Absolute_path = Reference_path & "\" & Relative_path
		If File.Exists (Absolute_path, "") Then
			Absolute_path = Get_long_file_name (Reference_path & "\" & Relative_path)
			Return Absolute_path
		Else
			Return ""
		End If
	Else
		Absolute_path = Get_long_file_name (Relative_path)
		Return Absolute_path
	End If
End Sub

' Returns true if the path is relative
Public Sub Is_relative_path (Path As String) As Boolean 
	If Path.Contains (":") Then
		Return False
	Else
		Return True
	End If
End Sub

' Get the long file name of a path
Public Sub Get_long_file_name (Path As String) As String
   Dim fileO As JavaObject
   fileO.InitializeNewInstance("java.io.File", Array As Object(Path))
   Return fileO.RunMethod("getCanonicalPath", Null)
End Sub

' Choose a file with FileChooser
Public Sub File_choose (Action As String, Title As String, Initial_file_name As String, Initial_folder As String, Extensions_description As String, Extensions_array () As String) As String 
	Dim File_chooser As FileChooser
	File_chooser.Initialize
	If File.Exists (Initial_folder, "") Then
		File_chooser.InitialDirectory = Initial_folder
	Else If File.Exists (Last_folder_choosed, "") Then
		File_chooser.InitialDirectory = Last_folder_choosed
	End If
	If File.Exists (Initial_file_name, "") Then
		File_chooser.InitialFileName = Initial_file_name 
	Else If File.Exists (Last_file_choosed, "") Then
		File_chooser.InitialFileName = Last_file_choosed
	End If
	File_chooser.SetExtensionFilter(Extensions_description, Extensions_array)
	File_chooser.Title = Title 
	Dim File_chooser_form As Form 
	File_chooser_form.Initialize ("", -1, -1)
	Select Case Action.ToUpperCase 
	Case "OPEN"
		Dim File_choosed As String = File_chooser.ShowOpen (File_chooser_form)
	Case "SAVE"
		Dim File_choosed As String = File_chooser.ShowSave (File_chooser_form)
	End Select
	If File_choosed.Trim <> "" Then Last_file_choosed = File_choosed : Last_folder_choosed = File.GetFileParent (File_choosed)
	Return File_choosed
End Sub

' Choose a folder with DirectoryChooser
Public Sub Folder_choose (Title As String, Initial_folder As String) As String 
	Dim Folder_chooser As DirectoryChooser
	Folder_chooser.Initialize
	If File.Exists (Initial_folder, "") Then
		Folder_chooser.InitialDirectory = Initial_folder
	Else If File.Exists (Last_folder_choosed, "") Then
		Folder_chooser.InitialDirectory = Last_folder_choosed
	End If
	Folder_chooser.Title = Title 
	Dim Folder_chooser_form As Form 
	Folder_chooser_form.Initialize ("", -1, -1)
	Dim Folder_choosed As String = Folder_chooser.Show (Folder_chooser_form)
	If Folder_choosed.Trim <> "" Then Last_folder_choosed = Folder_choosed
	Return Folder_choosed
End Sub

' Return "folder" or "file" string
Public Sub File_or_folder_type (Folder As String, File_name As String) As String 
	If File.IsDirectory (Folder, File_name) Then
		Return "folder"
	Else
		Return "file"
	End If
End Sub

' Create a new folder inside the folder
Public Sub Create_folder (Folder As String, New_folder As String)
	Try
		File.MakeDir (Folder, New_folder)
	Catch
		Dim Message As Messages : Message.Initialize (Me, "")
		Message.Message_type = Message.C_Toast_message
		Message.Show ("Create folder", "Impossible to create this folder : <br><br><I>" & _
								File.Combine (Folder, New_folder) & "</I><br><br>" & LastException.Message, Array As String ())
	End Try
End Sub

' Ask for a folder name and create it inside the folder
Public Sub Create_folder_interactively (Folder As String)
	Dim Message As Messages : Message.Initialize (Me, "")
	Message.Message_type = Message.C_Input_box
	If Message.Show ("Add folder", "Please type the folder name to create in : <br><br><I>" & Folder & "</I>", Array As String ("Create this folder", "Cancel")) = 0 Then
		If Message.Input_field.Text.Trim <> "" Then
			Create_folder (Folder, Message.Input_field.Text.Trim)
		End If
	End If
End Sub

' Delete a folder or a file
Public Sub Delete_folder_or_file (Folder As String, File_name As String)
	Dim Message As Messages : Message.Initialize (Me, "")
	Message.Message_type = Message.C_Toast_message
	If File.IsDirectory (Folder, File_name) = False Then
	Else If File.ListFiles (File.Combine (Folder, File_name)).Size > 0 Then
		Message.Show ("Delete folder", "The folder is not empty : <br><br>" & _
								File.Combine (Folder, File_name) & "<br><br>", Array As String ())
		Return
	End If
	Try
		File.Delete (Folder, File_name)
	Catch
		Dim Item_type As String = File_or_folder_type (Folder, File_name)
		Message.Show ("Delete " & Item_type, "Impossible to delete this " & Item_type & " : <br><br><I>" & _
								File.Combine (Folder, File_name) & "</I><br><br>" & LastException.Message, Array As String ())
	End Try
End Sub

' Ask for confirmation and delete a folder or a file
Public Sub Delete_folder_or_file_interactively (Folder As String, File_name As String)
	Dim Message As Messages : Message.Initialize (Me, "")
	Message.Message_type = Message.C_Message_box
	Dim Item_type As String = File_or_folder_type (Folder, File_name)
	If Message.Show ("Delete " & Item_type, "Do you really want to delete this " & Item_type & " : <br><br><I>" & File.Combine (Folder, File_name) & "</I>", _
							Array As String ("Yes, I want to delete the " & Item_type, "No, don't delete the " & Item_type)) = 0 Then
		Delete_folder_or_file (Folder, File_name)
	End If
End Sub

' Rename a folder or a file
Public Sub Rename_folder_or_file (Folder As String, File_name As String, New_Folder As String, New_name As String)
	Try
		' Credits to rwblinn - thank you Robert
		' Rename a file - Path can contain full and different path, which would be the same as moving a file
		' Class Reference: http://docs.oracle.com/javase/8/docs/api/java/io/File.html
	    Dim joFileSource As JavaObject
	    Dim joFileTarget As JavaObject
	    ' Create the file instances for the files (the constructor)
	    joFileSource.InitializeNewInstance("java.io.File", Array As Object(File.Combine (Folder, File_name)))
	    joFileTarget.InitializeNewInstance("java.io.File", Array As Object(File.Combine (New_Folder, New_name)))
	    ' Rename source file to target file
	    joFileSource.RunMethod("renameTo", Array As Object(joFileTarget))
	Catch
		Dim Message As Messages : Message.Initialize (Me, "")
		Message.Message_type = Message.C_Toast_message
		Dim Item_type As String = File_or_folder_type (Folder, File_name)
		Message.Show ("Rename " & Item_type, "Impossible to rename this " & Item_type & " : <br><br><I>" & _
								File.Combine (Folder, File_name) & " </I> into  <I><br><br>" & _
								File.Combine (New_Folder, New_name) & " </I><br><br>" & LastException.Message, Array As String ())
	End Try
End Sub

' Ask for a new folder or file name and rename the folder or file
Public Sub Rename_folder_or_file_interactively (Folder As String, File_name As String)
	Dim Message As Messages : Message.Initialize (Me, "")
	Message.Message_type = Message.C_Input_box
	Dim Item_type As String = File_or_folder_type (Folder, File_name)
	If Message.Show ("Rename " & Item_type, "Please type the new name of this " & Item_type & " : <br><br> <I>" & File.Combine (Folder, File_name) & "</I>", _
							Array As String ("Rename this " & Item_type, "Cancel")) = 0 Then
		If Message.Input_field.Text.Trim <> "" Then
			Rename_folder_or_file (Folder, File_name, Folder, Message.Input_field.Text.Trim)
		End If
	End If
End Sub
#End Region 

