﻿Type=Class
Version=2.2
B4J=true
@EndOfDesignText@
' Paste it around widget
' ###########################################################################################################################
' This widget is a part of One Click Around application : 	http://oneclickaround.esy.es/
' One Click Around forum : 		http://oneclickaround.esy.es/Forum

' Copyright (C) 2014 by Didier Terrien, France
' ###########################################################################################################################

Sub Class_Globals
	Private fx As JFX
	Private Calling_module As Object, Event_name As String 
	Private Owner As Pane 
	Public Style As String 
	Private Parameters_setup As CL_setup
	Public Widget_occupied As Boolean 
	Private Left, Top As Double 

	Private Visibility_delay As Int = 0
	Private With_notification As Boolean = True								' Notification when a note is selected
	Private Default_widget_folder As String = "\Widgets\Paste it around"	' Default DirApp relative path where to put notes
	Private Notes_popup As Popup_menu 
	Private Note_settings As Pane
	Private Edit_box As TextArea 
	Private Title As TextField
	Private B_Cancel As Button
	Private B_Save_exit As Button
	Private B_Copy_to_clipboard As Button
	Private B_Paste As Button
End Sub

' Initialize the widget
' Calling_module_v : module which called this sub
' Event_name_v : event preffix for callback subs
' Owner_v : the pane which will contain widget UI
Public Sub Initialize (Calling_module_v As Object, Event_name_v As String, Owner_v As Pane) 
	Calling_module = Calling_module_v 
	Event_name = Event_name_v 
	Owner = Owner_v
	Initialize_panel 
End Sub

Public Sub Launch (Left_v As Double, Top_v As Double, Width_v As Double, Height_v As Double, Commands As String)
	If Widget_occupied Then 
		Return
	Else
		Widget_occupied = True 
	End If
	Left = Left_v : Top = Top_v 
	If Commands.Trim = "?" Then 						' Empty commands
		Show_command_line_setup_window
		Return
	Else
		Dim Arguments () As String = Regex.Split (",", Commands)
		Dim Notes_folder As String = Arguments(0).Trim 
		If Notes_folder.Trim = "" Then Notes_folder = File.Combine (File.DirApp, Default_widget_folder)
		If File.Exists (Notes_folder, "") Then 						' If Notes_folder exists as an absolute path
		Else If File.Exists (File.DirApp, Notes_folder) Then 		' If Notes_folder exists as a DirApp relative path
			Notes_folder = File.Combine (File.DirApp, Notes_folder)
		Else														' If Notes_folder doesn't exist
			Notes_folder = File.Combine (File.DirApp, Default_widget_folder)
		End If
		If File.IsDirectory (Notes_folder, "") = False Then			' If Notes_folder is a file
			Notes_folder = File.GetFileParent (Notes_folder)
		End If
			
		Notes_folder = Utils.Get_long_file_name (Notes_folder)
		Notes_popup.Current_folder = Notes_folder
		Notes_popup.Navigation_root_folder = Notes_folder
		
		' Look for options
		For Each Argument As String In Arguments 
			If Argument.Trim.StartsWith ("/VD:") AND Argument.Trim.Length > 4 Then	' Option visibility delay
				Visibility_delay = Argument.Trim.SubString (4)
			Else If Argument.Trim = "/NN" Then										' Option no notification
				With_notification = False
			Else If Argument.Trim = "/HOC" Then										' Option to hide popup on click
				Notes_popup.Hide_on_click = True
			End If
		Next 

		Notes_popup.Show (Left_v, Top_v, Width_v, Height_v, Visibility_delay)
	End If
End Sub

' Reset widget
Public Sub Reset
	Widget_occupied = False
	If SubExists(Calling_module, Event_name & "_Reset_completed") Then
		CallSubDelayed (Calling_module, Event_name & "_Reset_completed")
	End If
End Sub


#Region  == Widget interface ==========================================================================
Private Sub	Initialize_panel 
	If Notes_popup.IsInitialized = False Then 
		Notes_popup.Initialize (Me, "Notes_popup", Null) 
		Notes_popup.Panel.Style = Utils.Init_default_style (Style)
		Notes_popup.Popup_type = Notes_popup.C_Folder_and_file_type
		Notes_popup.Hide_on_click = False 
		Notes_popup.With_navigation = True
		Notes_popup.B_Help_visible = True
		Notes_popup.B_Go_to_parent_visible = True
		Notes_popup.B_Add_visible = True
		Notes_popup.B_Delete_visible = True
		Notes_popup.B_Open_visible = True
		Notes_popup.Instructions = "With Paste it around, you add notes to a list." & CRLF & _
									"Then, when you left click a note inside the list," & CRLF & _
									"it will be pasted into clipboard." & CRLF & CRLF & _
									"Click '+' button to add a note" & CRLF & _
									"Left click a note to paste it into clipboard" & CRLF & _
									"Click 'O' button to open and modifiy a note" & CRLF & _
									"Click '-' button to delete a note" & CRLF & _
									"Left click a folder to see notes inside" & CRLF & _
									"Click upper arrow button to go to parent folder" & CRLF & CRLF & _
									"This widget is a part of One Click Around application :" & CRLF & _
									"http://oneclickaround.esy.es/" & CRLF
		Owner.AddNode(Notes_popup.Panel, 0, 0, 100, 100)
		Notes_popup.File_icon.Initialize (File.DirAssets, "Small file.png")
		Notes_popup.Folder_icon.Initialize (File.DirAssets, "Small folder.png")
	End If
End Sub

Public Sub Notes_popup_Left_click (Action As String, Clicked_button As String)
	If File.Exists (Notes_popup.Tree_title.Text, Action) = False Then
	Else If File.IsDirectory (Notes_popup.Tree_title.Text, Action) Then
	Else
		Dim Text As String = File.ReadString (Notes_popup.Tree_title.Text, Action)
		fx.Clipboard.SetString(Text)
		If With_notification Then
			If SubExists(Calling_module, Event_name & "_Left_click") Then
				CallSub2(Calling_module, Event_name & "_Left_click", Action)
			End If
		End If
		Reset
	End If
End Sub

Public Sub Notes_popup_B_Go_to_parent_clicked (EventData As MouseEvent)
End Sub

Public Sub Notes_popup_B_Add_clicked (EventData As MouseEvent)
	Show_setup_window ("")
End Sub

Public Sub Notes_popup_B_Delete_clicked (Selected_file As String, EventData As MouseEvent)
	Notes_popup.Delete_folder_or_file_interactively 
End Sub

Public Sub Notes_popup_B_Open_clicked (Selected_file As String, EventData As MouseEvent)
	Show_setup_window (Selected_file)
End Sub
 
Public Sub Notes_popup_Reset_completed
	Reset
End Sub
#End Region

#Region  == Command line interface ====================================================================
Private Sub Show_command_line_setup_window
	Parameters_setup.Initialize (Me, "Parameters_setup", Owner, "Paste it around widget setup")
	Parameters_setup.Style = Style
	Parameters_setup.Add_option ("Notes folder", Parameters_setup.C_Folder_type, "", Null, Me, "", "")
	Parameters_setup.Add_option ("Visibility delay (0 for no delay)", Parameters_setup.C_Text_type, "0", Null, Me, "", "/VD:")
	Parameters_setup.Add_option ("Notification", Parameters_setup.C_Combobox_type, "", Array As String ("With notification", "Without notification"), Me, "", "")
	Parameters_setup.Add_option ("Hide popup on click", Parameters_setup.C_Combobox_type, "", Array As String ("Hide popup menu on click", "Don't hide popup menu on click"), Me, "", "")
	Parameters_setup.Show (Left, Top)
End Sub

Private Sub Parameters_setup_Setup_ok (Output_command_line As String)
	Dim New_command_line As String = Output_command_line
	New_command_line = New_command_line.Replace ("With notification", "")
	New_command_line = New_command_line.Replace ("Without notification", "/NN")
	New_command_line = New_command_line.Replace ("Don't hide popup menu on click", "")
	New_command_line = New_command_line.Replace ("Hide popup menu on click", "/HOC")
	If SubExists(Calling_module, Event_name & "_Setup_ok") Then
		CallSub2(Calling_module, Event_name & "_Setup_ok", New_command_line)
	End If
	Reset
End Sub

Private Sub Parameters_setup_Setup_cancel 
	Close_setup_window
	Reset
End Sub
#End Region

#Region  == Widget setup interface ====================================================================
Private Sub Show_setup_window (Selected_file As String)
	Dim Note_text As String 
	If Selected_file.Trim = "" Then 												' New note
		Selected_file = "No name"
		Note_text = ""
	Else If File.Exists (Selected_file, "") = False Then	' Note file doesn't exist
		Return
	Else If File.IsDirectory (Selected_file, "") Then		' Folder was clicked
		Return
	Else
		Note_text = File.ReadString (Selected_file, "")
	End If
	Notes_popup.Tree.Enabled = False
	Note_settings.Initialize ("Note_settings")
	Note_settings.Style = Utils.Init_default_style (Style)
	
	Note_settings.LoadLayout ("Note")
	Owner.AddNode (Note_settings, Notes_popup.Panel.Left + Notes_popup.Panel.Width - 60, Notes_popup.Panel.Top, 300, 200)
	
	Title.Text = File.GetName (Selected_file)
	If Selected_file = "No name" Then 
		Title.Editable = True 
		Title.RequestFocus 
		Title.SelectAll 
	Else
		Title.Editable = False 
		Title.Enabled = False
	End If
	Edit_box.Text = Note_text
	B_Save_exit.TooltipText = "Save the text and exit settings window"
	B_Cancel.TooltipText = "Exit settings window without saving the text"
	B_Copy_to_clipboard.TooltipText = "Copy the text to clipboard"
	B_Paste.TooltipText = "Paste the text from clipboard"
End Sub

Private Sub Save_note_file
	Dim Text_to_save As String = Edit_box.Text
	Text_to_save.Replace (Chr (10), CRLF)
	Dim File_name As String 
	If Title.Text.Trim = "" Then
		File_name = "No name"
	Else
		File_name = Title.Text.Trim
	End If
	File.WriteString (Notes_popup.Tree_title.Text, File_name, Text_to_save)
	Notes_popup.Refresh_folder
End Sub

Private Sub Close_setup_window
	If Note_settings.IsInitialized Then 
		Note_settings.Visible = False
		Note_settings.RemoveNodeFromParent 
		Note_settings.Initialize ("")
		Note_settings = Null
	End If
	Notes_popup.Tree.Enabled = True
End Sub

' Buttons ------------------------------------------------------------------------------
Private Sub B_Save_exit_MouseClicked (EventData As MouseEvent)
	Save_note_file
	Close_setup_window
End Sub

Private Sub B_Cancel_MouseClicked (EventData As MouseEvent)
	Close_setup_window
End Sub

Sub B_Paste_MouseClicked (EventData As MouseEvent)
	Edit_box.Text = Edit_box.Text.SubString2 (0, Edit_box.SelectionStart) & fx.Clipboard.GetString & Edit_box.Text.SubString (Edit_box.SelectionEnd)
End Sub

Sub B_Copy_to_clipboard_MouseClicked (EventData As MouseEvent)
	fx.Clipboard.SetString (Edit_box.Text)
End Sub
#End Region 
